<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

define('MOONY_VERSION', '0.14.0');
define('MOONY_ROOT_DIR', dirname(__FILE__));

require_once MOONY_ROOT_DIR . '/Moony/Processor.php';

/**
 * Moonyのフロントエンドクラスです。
 * Moonyの動作設定を保持します。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony
{
    /**
     * アクションクラスを格納するディレクトリ
     * @var string
     */
    var $action_dir = 'actions';

    /**
     * テンプレートファイルを格納するディレクトリ
     * @var string
     */
    var $template_dir = 'templates';

    /**
     * キャッシュファイルを格納するディレクトリ
     * @var string
     */
    var $cache_dir;

    /**
     * 入力エンコーディング
     * @var string
     */
    var $input_encoding;

    /**
     * 出力エンコーディング
     * @var string
     */
    var $output_encoding;

    /**
     * 内部エンコーディング
     * @var string
     */
    var $internal_encoding;

    /**
     * セッションを自動的に開始するかどうか
     * @var bool
     */
    var $session_auto_start = true;

    /**
     * セッションのオプション設定の連想配列
     * @var array
     */
    var $session_options;

    /**
     * 処理を開始します。
     *
     * @access public
     */
    function start()
    {
        if (is_null($this->internal_encoding)) {
            // internal encoding
            $this->internal_encoding = mb_internal_encoding();
        }
        // check
        $this->_check();
        // start
        $processor =& new Moony_Processor();
        $processor->process($this);
    }

    /**
     * アクションクラスを格納するディレクトリを設定します。
     * 初期設定値は「actions」です。
     *
     * @access public
     * @param string $action_dir アクションクラスを格納するディレクトリ
     */
    function setActionDir($action_dir)
    {
        $this->action_dir = $action_dir;
    }

    /**
     * テンプレートファイルを格納するディレクトリを設定します。
     * 初期設定値は「templates」です。
     *
     * @access public
     * @param string $template_dir テンプレートファイルを格納するディレクトリ
     */
    function setTemplateDir($template_dir)
    {
        $this->template_dir = $template_dir;
    }

    /**
     * キャッシュファイルを格納するディレクトリを設定します。
     * 設定した場合、Moony_Cacheクラスがロードされ、
     * アクションクラス内で使用できるようになります。
     * 設定されない場合、Moony_Cacheクラスはロードされません。
     *
     * @access public
     * @param string $cache_dir キャッシュファイルを格納するディレクトリ
     */
    function setCacheDir($cache_dir)
    {
        $this->cache_dir = $cache_dir;
    }

    /**
     * 入力エンコーディングを設定します。
     * 設定した場合、リクエストパラメータの値は
     * 内部エンコーディングに変換されます。
     *
     * @access public
     * @param string $input_encoding 入力エンコーディング
     */
    function setInputEncoding($input_encoding)
    {
        $this->input_encoding = $input_encoding;
    }

    /**
     * 出力エンコーディングを設定します。
     * 設定した場合、出力時のデフォルトエンコーディングとして使用されます。
     *
     * @access public
     * @param string $output_encoding 出力エンコーディング
     */
    function setOutputEncoding($output_encoding)
    {
        $this->output_encoding = $output_encoding;
    }

    /**
     * 内部エンコーディングを設定します。
     * 設定されない場合、mb_internal_encodingの戻り値を
     * 内部エンコーディングとして使用します。
     *
     * @access public
     * @param string $internal_encoding 内部エンコーディング
     */
    function setInternalEncoding($internal_encoding)
    {
        $this->internal_encoding = $internal_encoding;
    }

    /**
     * セッションのオプション設定を設定します。
     * オプションには以下の指定が可能です:
     *   - name: セッション名
     *   - id: セッションID
     *   - save_path: セッションデータの保存先
     *   - cache_expire: カレントのキャッシュの有効期限
     *   - cache_limiter: カレントのキャッシュリミッタ
     *
     * @access public
     * @param array $options セッションのオプション設定
     */
    function setSessionOptions($options)
    {
        $this->session_options = $options;
    }

    /**
     * セッションを自動的に開始するかどうかを設定します。
     * 初期設定値は「true」で、変更しない限り
     * 自動的にセッションが開始されます。
     *
     * @access public
     * @param bool $session_auto_start セッションを自動で開始するかどうか
     */
    function setSessionAutoStart($session_auto_start)
    {
        $this->session_auto_start = (bool) $session_auto_start;
    }

    /**
     * 設定の確認を行います。
     * アクションクラスとテンプレートファイルを格納するディレクトリの
     * 設定、存在のチェックを行います。
     *
     * @access protected
     */
    function _check()
    {
        if (strlen($this->action_dir) == 0) {
            trigger_error('Not defined: action_dir', E_USER_ERROR);
            exit;
        }
        $this->action_dir = realpath($this->action_dir);
        if ($this->action_dir === false) {
            trigger_error('Not found: action_dir', E_USER_ERROR);
            exit;
        }
        if (strlen($this->template_dir) == 0) {
            trigger_error('Not defined: template_dir', E_USER_ERROR);
            exit;
        }
        $this->template_dir = realpath($this->template_dir);
        if ($this->template_dir === false) {
            trigger_error('Not found: template_dir', E_USER_ERROR);
            exit;
        }
    }
}
?>
