<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

require_once MOONY_ROOT_DIR . '/Moony/Action.php';
require_once MOONY_ROOT_DIR . '/Moony/Request.php';
require_once MOONY_ROOT_DIR . '/Moony/Router.php';
require_once MOONY_ROOT_DIR . '/Moony/Session.php';

/**
 * 処理を順番に行います。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Processor
{
    /**
     * 処理を以下の順番で行います。
     *   1. リクエスト、セッションを司るコンポーネントの生成
     *   2. ルーティング処理（Moony_Router）
     *   3. アクションクラスが見つからない場合、テンプレートを直接表示
     *   4. アクションクラスのメンバ変数にリクエストパラメータを展開
     *   5. アクションクラスのメソッドを順番に実行
     *
     * @access public
     * @param object $moony Moonyのインスタンス（動作設定）
     */
    function process(&$moony)
    {
        // request
        $request =& new Moony_Request();
        if (!ini_get('mbstring.encoding_translation') &&
            !is_null($moony->input_encoding)) {
            // convert encoding
            $request->convertEncoding(
                $moony->input_encoding, $moony->internal_encoding);
        }

        // session
        $session =& new Moony_Session();

        // routing
        $router =& new Moony_Router();
        $action = $router->route($request, $moony->action_dir);
        if ($action === false) {
            // not defined: action_dir
            trigger_error('Not defined: action_dir', E_USER_ERROR);
            exit;
        }
        if (is_null($action)) {
            // not found: action class
            // try to render a template directly
            $action =& $this->_createBlankAction();
        }

        // set to the action instance
        $action->moony = &$moony;
        $action->request = &$request;
        $action->session = &$session;
        if (!is_null($moony->cache_dir)) {
            // cache
            include_once MOONY_ROOT_DIR . '/Moony/Cache.php';
            $action->cache =& new Moony_Cache($moony->cache_dir);
        }

        $this->_extractParams($action);
        $this->_processAction($action);
    }

    /**
     * Moony_Actionを継承しただけのアクションクラスを生成、
     * インスタンスを取得します。
     *
     * @access protected
     * @return object アクションクラスのインスタンス
     */
    function &_createBlankAction()
    {
        $code = 'class BlankAction extends Moony_Action {}';
        eval($code);
        $action =& new BlankAction();
        return $action;
    }

    /**
     * リクエストパラメータを
     * アクションクラスのメンバ変数として展開します。
     * 名称が「_」で始まるパラメータと、「moony」「request」
     * 「session」「cache」「validator」という名称のパラメータは
     * 展開されません（Moonyの処理で使用するため）。
     *
     * @access protected
     * @param object $action アクションクラスのインスタンス
     */
    function _extractParams(&$action)
    {
        $params = $action->request->getAll();
        $prop_keys = array_keys(get_object_vars($action));
        $ignore_keys = array(
            'moony', 'request', 'session', 'cache', 'validator');

        foreach ($params as $name => $value) {
            if ($name[0] != '_' && !in_array($name, $ignore_keys) &&
                in_array($name, $prop_keys)) {
                $action->{$name} = $value;
            }
        }
    }

    /**
     * アクションの処理を行います。
     * 以下の順番で処理が実行されます。
     * 該当するアクションクラスのメソッドが存在しない場合、処理されません。
     *   1. prepareメソッド（アクションクラス）
     *     - 処理前に必要な準備処理を記述します
     *     - リクエストパラメータの変換処理とか
     *     - セッションに関するMoonyの動作設定変更とか
     *   2. セッションの設定および開始
     *     - 動作設定に基づいてセッションの設定および開始を行います
     *   3. validateメソッド（アクションクラス）
     *     - リクエストパラメータなどの検証処理を行います
     *   4. executeメソッド（アクションクラス）
     *     - 実際のメイン処理を記述します
     *
     * @access protected
     * @param object $action アクションクラスのインスタンス
     */
    function _processAction(&$action)
    {
        ob_start();

        // action::prepare
        if (method_exists($action, 'prepare')) {
            $action->prepare();
        }

        // session
        if (is_array($action->moony->session_options)) {
            // set options
            $this->_setSessionOptions(
                $action->session, $action->moony->session_options);
        }
        if ($action->moony->session_auto_start) {
            // start automatically
            $action->session->start();
        }

        // action::validate
        if (method_exists($action, 'validate')) {
            include_once MOONY_ROOT_DIR . '/Moony/Validator.php';
            $validator =&
                new Moony_Validator($action->moony->internal_encoding);
            $action->validator = &$validator;
            $action->validate();
        }

        // action::execute
        $action->execute();

        ob_end_flush();
    }

    /**
     * セッションに関するオプションを設定します。
     *
     * @access protected
     * @param object $session Moony_Sessionのインスタンス
     * @param array $session_options セッションのオプション設定
     */
    function _setSessionOptions(&$session, $session_options)
    {
        foreach ($session_options as $name => $value) {
            switch ($name) {
                case 'name':
                    $session->setName($value);
                    break;
                case 'id':
                    $session->setId($value);
                    break;
                case 'save_path':
                    $session->setSavePath($value);
                    break;
                case 'cache_expire':
                    $session->setCacheExpire($value);
                    break;
                case 'cache_limiter':
                    $session->setCacheLimiter($value);
                    break;
            }
        }
    }
}
?>
