<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

/**
 * PHP交じりのファイルをテンプレートとして扱うクラスです。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_View
{
    /**
     * アクションクラスのインスタンス
     * @var object
     */
    var $_action;

    /**
     * コンストラクタです。
     *
     * @access public
     * @param object $action アクションクラスのインスタンス
     */
    function Moony_View(&$action)
    {
        $this->_action = &$action;
    }

    /**
     * テンプレートの処理結果を取得します。
     * テンプレートファイルが存在しない場合、falseを返します。
     * アクションクラスのpublicなプロパティを
     * このクラスのインスタンスにコピーして利用可能にします。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @return string|bool テンプレートの処理結果
     */
    function fetch($template, $local_vars = null)
    {
        $template_dir = $this->_action->moony->template_dir;
        if (is_null($template_dir)) {
            trigger_error('Not defined: template_dir', E_USER_ERROR);
            return false;
        }

        $template = $template_dir . DIRECTORY_SEPARATOR . $template;
        if (!file_exists($template)) {
            return false;
        }

        foreach (get_object_vars($this->_action) as $name => $value) {
            if (is_object($value)) {
                $this->{$name} = &$value;
            } else {
                $this->{$name} = $value;
            }
        }

        if (is_array($local_vars)) {
            extract($local_vars, EXTR_SKIP);
        }

        $GLOBALS['_moony_action'] =& $this->_action;

        ob_start();
        ob_implicit_flush(false);
        include $template;
        $result = ob_get_contents();
        ob_end_clean();

        return $result;
    }
}

/**
 * 別のテンプレートファイルを読み込んで表示するヘルパー関数です。
 * 該当のテンプレートファイルが存在しない場合、何も処理しません。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php moony_include('foo.php'); ?>）
 *
 * @access public
 * @param string $template テンプレートファイル
 */
function moony_include($template)
{
    $action = &$GLOBALS['_moony_action'];
    $result = $action->fetch($template);
    if ($result !== false) {
        print($result);
    }
}

/**
 * 全ての検証エラーメッセージを表示するヘルパー関数です。
 * エラーメッセージが設定されていない場合、何も処理しません。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php moony_errors(); ?>）
 *
 * @access public
 * @param string $header 全体のヘッダ
 * @param string $footer 全体のフッタ
 * @param string $prefix 個々のメッセージの接頭辞
 * @param string $postfix 個々のメッセージの接尾辞
 */
function moony_errors(
    $header = '<ul>', $footer = '</ul>',
    $prefix = '<li>', $postfix = '</li>')
{
    $action = &$GLOBALS['_moony_action'];
    if (!is_null($action->validator)) {
        $errors = $action->validator->getErrors();
        if (count($errors) > 0) {
            print($header);
            foreach ($errors as $error) {
                print($prefix . $error . $postfix);
            }
            print($footer . "\n");
        }
    }
}

/**
 * 検証エラーの有無を判断するヘルパー関数です。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php if (moony_has_error()): ?>）
 *
 * @access public
 * @return bool 検証エラーの存在有無
 */
function moony_has_error()
{
    $action = &$GLOBALS['_moony_action'];
    if (!is_null($action->validator)) {
        return $action->validator->hasError();
    }
    return false;
}

/**
 * ワンタイムトークンを埋め込んだ
 * hiddenフィールドを埋め込むヘルパー関数です。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php moony_token(); ?>）
 *
 * @access public
 * @param bool $is_xhtml XHTMLとして埋め込むかどうか
 */
function moony_token($is_xhtml = true)
{
    $action = &$GLOBALS['_moony_action'];
    $token = md5(uniqid(rand(), true));
    $action->session->set('_moony_token', $token);
    $html = '<input type="hidden" name="_moony_token" value="'
          . $token
          . ($is_xhtml ? '" />' : '">') . "\n";
    print($html);
}

/**
 * 文字列を出力用にエスケープするヘルパー関数です。
 * PHPテンプレートの内部で使用されます。
 * （例: <?php h($this->foo); ?>）
 * $echoがtrueの場合、エスケープ結果は出力されます。
 * $echoがfalseの場合、エスケープ結果は戻り値として返されます。
 *
 * @access public
 * @param string $str エスケープ対象の文字列
 * @param bool $echo エスケープ結果を出力するかどうか
 * @return null|string $echoがtrueの場合、エスケープ結果
 */
function h($str, $echo = true)
{
    $action = &$GLOBALS['_moony_action'];
    $str = htmlspecialchars(
        $str, ENT_QUOTES, $action->moony->internal_encoding);
    if ($echo) {
        print($str);
    } else {
        return $str;
    }
}
?>
